"""
Download a youtube video as an mp3.
"""

import os
import shutil
import subprocess
import tempfile
import warnings

from docker_run_cmd.api import docker_run
from static_ffmpeg import add_paths


def yt_dlp_download_mp3(url: str, outmp3: str) -> None:
    """Download the youtube video as an mp3."""
    add_paths()
    par_dir = os.path.dirname(outmp3)
    if par_dir:
        os.makedirs(par_dir, exist_ok=True)

    with tempfile.TemporaryDirectory() as temp_dir:
        temp_file = os.path.join(temp_dir, "temp.mp3")
        for _ in range(3):
            try:
                cmd_list: list[str] = [
                    "yt-dlp",
                    url,
                    "--extract-audio",
                    "--audio-format",
                    "mp3",
                    "--output",
                    temp_file,
                ]
                subprocess.run(cmd_list, check=True)
                shutil.copy(temp_file, outmp3)
                return
            except subprocess.CalledProcessError as cpe:
                print(f"Failed to download {url} as mp3: {cpe}")
                continue
        warnings.warn(f"Failed all attempts to download {url} as mp3.")


def docker_yt_dlp_download_mp3(url: str, outmp3: str) -> None:
    """Download the youtube video as an mp3."""
    here = os.path.abspath(os.path.dirname(__file__))
    dockerfile = os.path.join(here, "Dockerfile")
    dockerfile = os.path.abspath(dockerfile)
    assert os.path.exists(dockerfile), f"dockerfile {dockerfile} does not exist"
    with tempfile.TemporaryDirectory() as temp_dir:
        cmd_args = [url, "--extract-audio", "--audio-format", "mp3", "--output", "/host_dir/temp.mp3", "--update", "--no-geo-bypass"]
        docker_run(name="yt-dlp", dockerfile_or_url=dockerfile, cwd=temp_dir, cmd_list=cmd_args)
        shutil.copy(os.path.join(temp_dir, "temp.mp3"), outmp3)


def download_mp3(url: str, outmp3: str) -> None:
    """Download the youtube video as an mp3."""
    docker_yt_dlp = os.environ.get("USE_DOCKER_YT_DLP", "0") == "1"
    if docker_yt_dlp:
        return docker_yt_dlp_download_mp3(url, outmp3)
    return yt_dlp_download_mp3(url, outmp3)



"""
Testing for io
"""

# pylint: disable=line-too-long,missing-function-docstring,missing-class-docstring,super-with-arguments,invalid-name,R0801

import gzip
import os
import shutil
import tempfile
import unittest

from vidcrawler.io import write_json


class IoTester(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cleanup = []

    def tearDown(self):
        for cleanup in self.cleanup:
            try:
                cleanup()
            except BaseException as be:  # pylint: disable=broad-except
                print(str(be))
        self.cleanup = []

    def create_tmp_dir(self) -> str:
        tmp_dir = os.path.join(
            tempfile.gettempdir(),
            "blast.video",
            "test",
            "io",
            os.urandom(24).hex(),
        )
        os.makedirs(tmp_dir, exist_ok=True)
        self.cleanup.append(lambda: shutil.rmtree(tmp_dir, ignore_errors=True))
        return tmp_dir

    def test_make_zip(self):
        tmp_dir = self.create_tmp_dir()
        json_content = {"key": "value"}
        json_path = os.path.join(tmp_dir, "my.json")
        write_json(json_path, content=json_content, also_gzip=True, minify=True)
        self.assertTrue(os.path.isfile(json_path), json_path)
        # Expect that zip file exists.
        zip_path = json_path + ".gz"
        self.assertTrue(os.path.isfile(zip_path), f"Path: {zip_path}")
        with gzip.open(zip_path, "rb") as f:
            data_reread = f.read().decode("utf-8")
        self.assertEqual(data_reread, '{"key": "value"}')


if __name__ == "__main__":
    unittest.main()



"""
Command entry point.
"""

# pylint: disable=consider-using-f-string

import argparse
import os

from vidcrawler.library import Library, VidEntry
from vidcrawler.youtube_bot import fetch_all_vids


def parse_args() -> argparse.Namespace:
    """Parse command line arguments."""
    parser = argparse.ArgumentParser("youtube-pull")
    parser.add_argument(
        "channel",
        type=str,
        # help="URL of the channel, example: https://www.youtube.com/@silverguru/videos",
        help="URL slug of the channel, example: @silverguru",
    )
    parser.add_argument("basedir", type=str)
    parser.add_argument(
        "--limit-scroll-pages",
        type=int,
        default=1000,
        help="Limit the number of the number of pages to scroll down",
    )
    parser.add_argument(
        "--download",
        action="store_true",
        help="Deprecated: This option does nothing.",
    )
    parser.add_argument(
        "--skip-download",
        action="store_true",
        help="Skip the download of the videos.",
    )
    parser.add_argument(
        "--download-limit",
        type=int,
        default=-1,
        help="Limit the number of videos to download",
    )
    parser.add_argument(
        "--skip-scan",
        action="store_true",
        help="Skip the update of the library.json file",
    )
    parser.add_argument(
        "--yt-dlp-uses-docker",
        action="store_true",
        help="Use docker to run yt-dlp",
    )
    return parser.parse_args()


def to_channel_url(channel: str) -> str:
    """Convert channel name to channel URL."""
    out = f"https://www.youtube.com/{channel}/videos"
    return out


def main() -> None:
    """Main function."""
    args = parse_args()
    if args.yt_dlp_uses_docker:
        os.environ["USE_DOCKER_YT_DLP"] = "1"
    channel_url = to_channel_url(args.channel)
    output_dir = os.path.join(args.basedir, args.channel, "youtube")
    limit_scroll_pages = args.limit_scroll_pages
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    library_json = os.path.join(output_dir, "library.json")
    library = Library(library_json)
    if not args.skip_scan:
        vids: list[VidEntry] = fetch_all_vids(channel_url, limit=limit_scroll_pages)
        library.merge(vids)
        print(f"Updated {library_json}")
    else:
        if not os.path.exists(library_json):
            raise FileNotFoundError(f"{library_json} does not exist. Cannot skip scan.")
    if args.download:
        print("Warning: The --download option is deprecated is now implied. Use --skip-download to avoid downloading")
    if not args.skip_download:
        library.download_missing(args.download_limit)


if __name__ == "__main__":
    import sys

    sys.argv.append("@silverguru")
    sys.argv.append("tmp")
    sys.argv.append("--limit-scroll-pages")
    sys.argv.append("1")
    sys.argv.append("--download-limit")
    sys.argv.append("1")
    main()



"""
    Date utility functions.
"""

# pylint: disable=invalid-name

from datetime import datetime
from re import findall
from typing import Optional, Union

import pytz
from dateutil.parser import parse

CURRENT_TIME_ZONE_STR = "America/Los_Angeles"


def now_local(tz_str: Optional[str] = None) -> datetime:
    """Returns timezone aware now, which allows subtractions"""
    tz_str = tz_str or CURRENT_TIME_ZONE_STR
    tz = pytz.timezone(tz_str)
    ts = datetime.now().astimezone(tz)
    return ts


def _my_date_parse(date_string: str) -> datetime:
    try:
        return datetime.fromisoformat(date_string)
    except ValueError as verr:
        if "Invalid isoformat" in str(verr):
            return parse(date_string, fuzzy=True)
        raise


def parse_datetime(s, tzinfo=None) -> datetime:  # type: ignore
    """Parses a datetime string"""
    date: datetime = _my_date_parse(s)
    if tzinfo:
        if isinstance(tzinfo, str):
            tzinfo = pytz.timezone(tzinfo)
        if date.tzinfo is None:
            date = date.replace(tzinfo=tzinfo)
        else:
            date = date.astimezone(tzinfo)
    return date


def iso_fmt(date_obj: Union[datetime, str]) -> str:
    """Outputs the ogbject as an iso formatted string"""
    if isinstance(date_obj, datetime):
        return date_obj.isoformat()
    if isinstance(date_obj, str):
        return parse_datetime(date_obj).isoformat()
    raise ValueError(f"{__file__} Unexpected type: {type(date_obj)}")


def iso8601_duration_as_seconds(d: str) -> int:
    """assert iso8601_duration_as_seconds('PT25M0S') == 1500"""
    if d[0] != "P":
        raise ValueError("Not an ISO 8601 Duration string")
    seconds = 0
    # split by the 'T'
    for i, item in enumerate(d.split("T")):
        for number, unit in findall(r"(?P<number>\d+)(?P<period>S|M|H|D|W|Y)", item):
            # print '%s -> %s %s' % (d, number, unit )
            number = int(number)
            this = 0
            if unit == "Y":
                this = number * 31557600  # 365.25
            elif unit == "W":
                this = number * 604800
            elif unit == "D":
                this = number * 86400
            elif unit == "H":
                this = number * 3600
            elif unit == "M":
                # ambiguity ellivated with index i
                if i == 0:
                    this = number * 2678400  # assume 30 days
                    # print "MONTH!"
                else:
                    this = number * 60
            elif unit == "S":
                this = number
            seconds = seconds + this
    return seconds



"""
VideoInfo object used to store information about a video.
"""

# pylint: disable=line-too-long,missing-function-docstring,consider-using-f-string,too-many-locals,invalid-name,no-else-return,too-many-instance-attributes

from __future__ import annotations

import json
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import Any, Dict, List, Optional, Set

from .date import iso_fmt, now_local, parse_datetime


@dataclass
class VideoInfo:
    """Object used to hold information about a video."""

    channel_name: str = ""
    title: str = ""
    date_published: str = ""  # from the scraped website
    date_discovered: str = ""  # generated during scrape
    date_lastupdated: str = ""
    channel_url: str = ""
    source: str = ""
    url: str = ""
    duration: str = ""  # units = seconds.
    description: str = ""
    img_src: str = ""
    img_status: int = -1
    img_width: int = -1
    img_height: int = -1
    iframe_src: str = ""
    views: str = ""
    profile_img_src: str = ""
    subtitles_url: str = ""
    rank: Optional[float] = None  # optional stdev rank.

    def to_dict(self) -> Dict:
        """Generates a dictionary representing this class instance."""
        out = {
            "channel_name": self.channel_name,
            "title": self.title,
            "date_published": self.date_published,
            "date_discovered": self.date_discovered,
            "date_lastupdated": self.date_lastupdated,
            "channel_url": self.channel_url,
            "source": self.source,
            "url": self.url,
            "duration": self.duration,
            "description": self.description,
            "img_src": self.img_src,
            "img_status": self.img_status,
            "img_width": self.img_width,
            "img_height": self.img_height,
            "iframe_src": self.iframe_src,
            "views": self.views,
            "profile_img_src": self.profile_img_src,
            "subtitles_url": self.subtitles_url,
        }
        if self.rank is not None:
            # parse float to two decimal places
            out["rank"] = round(self.rank, 2)
        return out

    @classmethod
    def from_json_str(cls, data: str) -> VideoInfo:
        """Deserializes from dictionary json str back to VideoInfo"""
        d = json.loads(data)
        return VideoInfo.from_dict(d)

    @classmethod
    def from_dict(cls, data: Dict) -> VideoInfo:
        """Deserializes from dictionary back to VideoInfo"""
        views = _parse_views(data["views"])
        return VideoInfo(
            channel_name=data["channel_name"],
            title=data["title"],
            date_published=iso_fmt(data["date_published"]),
            date_discovered=iso_fmt(data["date_discovered"]),
            date_lastupdated=iso_fmt(data["date_lastupdated"]),
            channel_url=data["channel_url"],
            source=data["source"],
            url=data["url"],
            duration=data["duration"],
            description=data["description"],
            img_src=data["img_src"],
            # img_width, img_height, status were added recently, and therefore are optional.
            img_status=data.get("img_status", -1),
            img_width=data.get("img_width", -1),
            img_height=data.get("img_height", -1),
            iframe_src=data["iframe_src"],
            views=views,
            profile_img_src=data["profile_img_src"],
            subtitles_url=data.get("subtitles_url", ""),
            rank=data.get("rank", None),
        )

    @classmethod
    def from_list_of_dicts(cls, data: List[Dict]) -> List[VideoInfo]:
        out: List[VideoInfo] = []
        for datum in data:
            vi = VideoInfo.from_dict(datum)
            out.append(vi)
        return out

    @classmethod
    def to_plain_list(cls, data: List[VideoInfo]) -> List[Dict]:
        out = []
        vid_info: VideoInfo
        for vid_info in data:
            d = vid_info.to_dict()
            out.append(d)
        return out

    @classmethod
    def to_compact_csv(
        cls,
        vid_list: List[VideoInfo],
        exclude_columns: Optional[Set[str]] = None,
    ) -> List[List]:
        """
        Generates a compact csv form of the data. The csv form consists of a header list, followed by N data
        lists. This has the advantage of eliminating the redundant keys in the dictionary form, which helps
        reduce the size of the file over the wire.
        """
        columns_set: Set[str] = set({})
        for vid in vid_list:
            vid_dict = vid.to_dict()
            for key in vid_dict.keys():
                columns_set.add(key)
        if exclude_columns:
            columns_set = columns_set - exclude_columns
        exclude_columns = exclude_columns or set({})
        columns = sorted(list(columns_set))
        out: List[Any] = []
        # Create the header for the csv
        out.append(columns)
        for vid in vid_list:
            vid_dict = vid.to_dict()
            row = []
            for col in columns:
                val = vid_dict.get(col, "")
                row.append(val)
            out.append(row)
        return out

    @classmethod
    def from_compact_csv(cls, csv_list: List[List]) -> List[VideoInfo]:
        """
        Generates a compact csv form of the data. The csv form consists of a header list, followed by N data
        lists. This has the advantage of eliminating the redundant keys in the dictionary form, which helps
        reduce the size of the file over the wire.
        """
        if len(csv_list) == 0:
            return []
        # Header line is the key list
        key_list: List = [e.strip() for e in csv_list[0]]
        out: List[VideoInfo] = []
        # Go through the rest of the list and and construct the data, using the key_list
        # as the schema to build out the output list of VideoInfo.
        for datum in csv_list[1:]:
            vi: VideoInfo = VideoInfo()
            d = vi.to_dict()
            for i, key in enumerate(key_list):
                d[key] = datum[i]
            out.append(VideoInfo.from_dict(d))
        return out

    def video_age_seconds(self, now_time: Optional[datetime] = None) -> float:
        """
        Returns the date published as a datetime object.
        """
        now_time = now_time or now_local()
        diff: timedelta = now_time - parse_datetime(self.date_published)
        return diff.total_seconds()

    # Set operations like intersection, union, etc are nice to have.
    # This needs hash, eq, and ne.
    def __hash__(self):
        return hash(self.channel_name + self.title)

    def __eq__(self, other):
        return (self.channel_name, self.title) == (
            other.channel_name,
            other.title,
        )

    def __ne__(self, other):
        # Not strictly necessary, but to avoid having both x==y and x!=y
        # True at the same time
        return not (self == other)  # pylint: disable=superfluous-parens


def _parse_views(view_str: str) -> str:
    try:
        if view_str == "?":
            return view_str
        if view_str == "":
            return "0"
        multiplier = 1
        view_str = str(view_str).upper().replace(",", "")
        if "K" in view_str:
            view_str = view_str.replace("K", "")
            multiplier = 1000
        elif "M" in view_str:
            view_str = view_str.replace("M", "")
            multiplier = 1000 * 1000
    except BaseException as e:  # pylint: disable=broad-except
        print(e)
    return str(int(float(view_str) * multiplier))


def test() -> None:
    url: str = "2021-03-29 01:13:15+00:00"
    d: Any = parse_datetime(url)
    print(f'"{d.isoformat()}"')


if __name__ == "__main__":
    test()



# pylint: disable=too-many-locals

"""
Scrapes the brighteon website for video urls and downloads them.
"""

import argparse
import os
import sys
import warnings

from playwright.sync_api import Page

from vidcrawler.library import Library, VidEntry
from vidcrawler.playwright_launcher import launch_playwright, set_headless

BASE_URL = "https://www.brighteon.com"

INSTALLED = False


def _fetch_vid_infos(page: Page, channel_url: str, page_num: int) -> list[VidEntry]:
    """Get the urls from a channel page. Throws exception when page not found."""
    # From: https://www.brighteon.com/channels/hrreport
    # To: https://www.brighteon.com/channels/hrreport/videos?page=1
    url = f"{channel_url}/videos?page={page_num}"
    print(f"Fetching video list from {url}")
    response = page.goto(url)
    assert response
    if response.status != 200:
        raise ValueError(f"Failed to fetch {url}, status: {response.status}")
    # get all div class="post" objects
    posts = page.query_selector_all("div.post")
    # get the first one
    vids: list[VidEntry] = []
    for post in posts:
        try:
            link = post.query_selector("a")
            assert link
            href = link.get_attribute("href")
            assert href
            title = post.query_selector("div.title")
            assert title
            title_text = title.inner_text().strip()
            href = BASE_URL + href
            vids.append(VidEntry(title=title_text, url=href))
        except Exception as e:  # pylint: disable=broad-except
            warnings.warn(f"Failed to get url: {e}")
    print(f"Found {len(vids)} videos.")
    return vids


def _update_library(outdir: str, channel_name: str, full_scan: bool, limit: int = -1) -> Library:
    """Simple test to verify the title of a page."""
    channel_url = f"https://www.brighteon.com/channels/{channel_name}"
    library_json = os.path.join(outdir, "library.json")
    library = Library(library_json)
    count = 0
    stored_vids: list[VidEntry] = library.load()
    with launch_playwright() as (page, _):
        # Determine whether to run headless based on the environment variable
        urls: list[VidEntry] = []
        page_num = 0
        while True:
            if limit > -1:
                if count >= limit:
                    break
            count += 1
            try:
                new_urls: list[VidEntry] = _fetch_vid_infos(page, channel_url, page_num)
                set_new_urls = set(new_urls)
                set_stored_vids = set(stored_vids)
                # if the new urls are fully contained in the stored vids, then we are done
                if not full_scan and (set_new_urls <= set_stored_vids):
                    warnings.warn("All the new videos are already in the library... halting scan.")
                    break
                page_num += 1
                urls += new_urls
            except Exception as e:  # pylint: disable=broad-except
                warnings.warn(f"Failed to get urls: {e}")
                break
    print(f"Got {len(urls)} urls.")
    library.merge(urls)
    return library


def main() -> int:
    """Main function."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--channel-name",
        type=str,
        help="URL slug of the channel, example: hrreport",
        required=True,
    )
    parser.add_argument("--output", type=str, help="Output directory", required=True)
    parser.add_argument(
        "--limit-downloads",
        type=int,
        default=-1,
    )
    parser.add_argument(
        "--skip-download",
        action="store_true",
    )
    parser.add_argument(
        "--yt-dlp-uses-docker",
        action="store_true",
        help="Use docker to run yt-dlp",
    )
    set_headless(True)
    # full-scan
    parser.add_argument("--full-scan", action="store_true", help="Scan the entire channel, not just the new videos.")
    args = parser.parse_args()
    if args.yt_dlp_uses_docker:
        os.environ["USE_DOCKER_YT_DLP"] = "1"
    outdir = args.output
    channel = args.channel_name
    download_limit = args.limit_downloads
    skip_download = args.skip_download
    full_scan = args.full_scan

    library = _update_library(outdir, channel, full_scan=full_scan, limit=download_limit)
    if not skip_download:
        library.download_missing(download_limit)
    return 0


def unit_test(limit=-1) -> int:
    """Run the tests."""
    sys.argv.append("--channel-name")
    sys.argv.append("hrreport")
    sys.argv.append("--output")
    sys.argv.append("tmp2")
    sys.argv.append("--yt-dlp-uses-docker")
    sys.argv.append("--limit-downloads")
    sys.argv.append(str(limit))
    main()
    return 0


if __name__ == "__main__":
    sys.exit(unit_test(1))



# pylint: disable=missing-function-docstring
# pylint: disable=consider-using-f-string
# pylint: disable=missing-module-docstring

import os
import sys
from shutil import rmtree

from setuptools import Command, find_packages, setup

# The directory containing this file
HERE = os.path.dirname(__file__)
NAME = "vidcrawler"
URL = "https://github.com/zackees/vidcrawler"
EMAIL = "dont@email.me"
AUTHOR = "Zach Vorhies"
REQUIRES_PYTHON = ">=3.6.0"
VERSION = None

# The text of the README file
with open(os.path.join(HERE, "README.md"), encoding="utf-8", mode="rt") as fd:
    LONG_DESCRIPTION = fd.read()

with open(
    os.path.join(HERE, "requirements.txt"), encoding="utf-8", mode="rt"
) as fd:
    REQUIREMENTS = [line.strip() for line in fd.readlines() if line.strip()]


def get_requirements():
    with open(
        os.path.join(HERE, "requirements.txt"), encoding="utf-8", mode="rt"
    ) as f:
        packages = []
        for line in f:
            line = line.strip()
            # let's also ignore empty lines and comments
            if not line or line.startswith("#"):
                continue
            if "https://" in line:
                tail = line.rsplit("/", 1)[1]
                tail = tail.split("#")[0]
                line = tail.replace("@", "==").replace(".git", "")
            packages.append(line)
    return packages


def get_version() -> str:
    version_file = os.path.join(HERE, NAME, "version.py")
    with open(version_file, encoding="utf-8", mode="rt") as fd:
        for line in fd.readlines():
            if line.startswith("VERSION"):
                VERSION = line.split("=")[1].strip().strip('"')
                return VERSION
    raise RuntimeError("Could not find version")


VERSION = get_version()


class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        pass

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(HERE, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution…")

        def exe(cmd: str) -> None:
            print(f"Executing:\n  {cmd}\n")
            return os.system(cmd)

        exe(f'"{sys.executable}" setup.py sdist bdist_wheel --universal')

        self.status("Uploading the package to PyPI via Twine…")
        if 0 != exe("twine upload dist/*"):
            raise RuntimeError("Upload failed")

        self.status("Pushing git tags…")
        exe(f"git tag v{VERSION} && git push --tags")

        sys.exit()


setup(
    python_requires=REQUIRES_PYTHON,
    version=VERSION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url=URL,
    author="Zach Vorhies",
    author_email="dont@email.me",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.9",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Operating System :: MacOS :: MacOS X",
        "Environment :: Console",
    ],
    install_requires=REQUIREMENTS,
    packages=find_packages(
        exclude=["tests", "*.tests", "*.tests.*", "tests.*"]
    ),
    package_data={},
    include_package_data=True,
    cmdclass={
        "upload": UploadCommand,
    },
)



# pylint: disable=all
import os
import tempfile
import unittest
from typing import List

import requests

from vidcrawler.fetch_html import fetch_html
from vidcrawler.video_info import VideoInfo
from vidcrawler.youtube import fetch_youtube_today, parse_youtube_video


class YouTubeScraperTester(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cleanup = []

    def tearDown(self):
        for cleanup in self.cleanup:
            try:
                cleanup()
            except BaseException as be:  # pylint: disable=broad-except
                print(str(be))
        self.cleanup = []

    def create_tempfile_path(self) -> str:
        tmp_file = tempfile.NamedTemporaryFile(suffix=".sqlite3", delete=False)
        tmp_file.close()
        self.cleanup.append(lambda: os.remove(tmp_file.name))
        return os.path.abspath(tmp_file.name)

    @unittest.skip("test_fetch_video_info not working")
    def test_fetch_video_info(self) -> None:
        url: str = "https://www.youtube.com/watch?v=5nBqIK0mrFI"
        content: str = fetch_html(url)
        d = parse_youtube_video(content)
        self.assertEqual("False", d["is_live"])
        self.assertEqual(
            "https://yt3.ggpht.com/ytc/AMLnZu8YcoqIA7-FfOC0tEwMjTDflDpiMjnJT6GvCXjunw=s176-c-k-c0x00ffffff-no-rj",
            d["profile_thumbnail"],
        )

    def test_bad_content(self) -> None:
        """Stephan Molynuex is banned. This should produce an error."""
        try:
            fetch_youtube_today(
                channel_name="Stefan Molynuex",
                channel_id="UCC3L8QaxqEGUiBC252GHy3w",
                cache_path=None,
            )
        except OSError:
            return  # expected result.
        self.fail("Expected bad channel to raise exception.")

    def test_maryam_henein(self) -> None:
        out: List[VideoInfo] = fetch_youtube_today(
            channel_name="Maryam Henein",
            channel_id="UCkw3pE7PwWfQCFFP2YgKVIQ",
            cache_path=None,
            limit=1,
        )
        self.assertEqual(len(out), 1)

    def test_lex_freemond(self) -> None:
        out: List[VideoInfo] = fetch_youtube_today(
            channel_name="Lex Fridman",
            channel_id="UCSHZKyawb77ixDdsGog4iWA",
            cache_path=None,
            limit=1,
        )
        self.assertEqual(len(out), 1)

    def test_fetch_channel_via_html(self) -> None:
        out = fetch_youtube_today(
            channel_name="Maryam Henein",
            channel_id="UCkw3pE7PwWfQCFFP2YgKVIQ",
            cache_path=None,
            limit=1,
        )
        self.assertEqual(len(out), 1)

    def test_video_xml_api(self):
        """Tests a major entry point used by the scraper to get YT vids."""
        # Links to tim pool channel.
        url = "https://www.youtube.com/feeds/videos.xml?channel_id=UCG749Dj4V2fKa143f8sE60Q"
        response = requests.get(url)
        # Raise error if 404 or other error condition.
        response.raise_for_status()

    def test_charlie_kirk(self):
        """Tests a major entry point used by the scraper to get YT vids."""
        # Links to tim pool channel.
        channel_id = "UCfaIu2jO-fppCQV_lchCRIQ"
        vids = fetch_youtube_today("Charlie Kirk", channel_id, None, 3)
        print(vids)
        # response = requests.get(url)
        # Raise error if 404 or other error condition.
        # response.raise_for_status()


if __name__ == "__main__":
    unittest.main()
